@@ -1,16 +1,16 @@
-// Copyright (c) 2019-2021, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2022, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
 
-// This file has been migrated from cmd/singularity/pull_test.go
+// The E2E PULL group tests image pulls of SIF format images (library, oras
+// sources). Docker / OCI image pull is tested as part of the DOCKER E2E group.
 
 package pull
 
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -41,6 +41,7 @@ type testStruct struct {
 	setImagePath     bool   // pass destination path
 	setPullDir       bool   // pass --dir
 	expectedExitCode int
+	workDir          string
 	pullDir          string
 	imagePath        string
 	expectedImage    string
@@ -135,14 +136,6 @@ var tests = []testStruct{
 		unauthenticated:  true,
 		expectedExitCode: 0,
 	},
-
-	{
-		desc:             "image from docker",
-		srcURI:           "docker://alpine:3.8",
-		force:            true,
-		unauthenticated:  false,
-		expectedExitCode: 0,
-	},
 	// TODO(mem): reenable this; disabled while shub is down
 	// {
 	// 	desc:            "image from shub",
@@ -220,6 +213,12 @@ var tests = []testStruct{
 }
 
 func (c *ctx) imagePull(t *testing.T, tt testStruct) {
+	// Use a one-time cache directory specific to this pull. This ensures we are always
+	// testing an entire pull operation, performing the download into an empty cache.
+	cacheDir, cleanup := e2e.MakeCacheDir(t, "")
+	defer cleanup(t)
+	c.env.UnprivCacheDir = cacheDir
+
 	// We use a string rather than a slice of strings to avoid having an empty
 	// element in the slice, which would cause the command to fail, without
 	// over-complicating the code.
@@ -245,6 +244,14 @@ func (c *ctx) imagePull(t *testing.T, tt testStruct) {
 		argv += tt.imagePath + " "
 	}
 
+	if tt.workDir == "" {
+		wd, err := os.Getwd()
+		if err != nil {
+			t.Fatalf("unable to get working directory: %s", err)
+		}
+		tt.workDir = wd
+	}
+
 	argv += tt.srcURI
 
 	c.env.RunSingularity(
@@ -252,6 +259,7 @@ func (c *ctx) imagePull(t *testing.T, tt testStruct) {
 		e2e.AsSubtest(tt.desc),
 		e2e.WithProfile(e2e.UserProfile),
 		e2e.WithEnv(tt.envVars),
+		e2e.WithDir(tt.workDir),
 		e2e.WithCommand("pull"),
 		e2e.WithArgs(strings.Split(argv, " ")...),
 		e2e.ExpectExit(tt.expectedExitCode))
@@ -277,7 +285,7 @@ func (c *ctx) setup(t *testing.T) {
 	e2e.EnsureRegistry(t)
 
 	// setup file and dir to use as invalid images
-	orasInvalidDir, err := ioutil.TempDir(c.env.TestDir, "oras_push_dir-")
+	orasInvalidDir, err := os.MkdirTemp(c.env.TestDir, "oras_push_dir-")
 	if err != nil {
 		t.Fatalf("unable to create src dir for push tests: %v", err)
 	}
@@ -328,14 +336,14 @@ func (c *ctx) setup(t *testing.T) {
 func (c ctx) testPullCmd(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
-			tmpdir, err := ioutil.TempDir(c.env.TestDir, "pull_test.")
+			tmpdir, err := os.MkdirTemp(c.env.TestDir, "pull_test.")
 			if err != nil {
 				t.Fatalf("Failed to create temporary directory for pull test: %+v", err)
 			}
 			defer os.RemoveAll(tmpdir)
 
 			if tt.setPullDir {
-				tt.pullDir, err = ioutil.TempDir(tmpdir, "pull_dir.")
+				tt.pullDir, err = os.MkdirTemp(tmpdir, "pull_dir.")
 				if err != nil {
 					t.Fatalf("Failed to create temporary directory for pull dir: %+v", err)
 				}
@@ -345,26 +353,17 @@ func (c ctx) testPullCmd(t *testing.T) {
 				tt.imagePath = filepath.Join(tmpdir, "image.sif")
 				tt.expectedImage = tt.imagePath
 			} else {
-				// Since we are not passing an image name, change the current
-				// working directory to the temporary directory we just created so
-				// that we know it's clean. We don't do this for the other case in
-				// order to catch spurious files showing up. Maybe later we can
-				// examine the directory and assert that it only contains what we
-				// expect.
-				oldwd, err := os.Getwd()
-				if err != nil {
-					t.Fatalf("Failed to get working directory for pull test: %+v", err)
-				}
-				defer os.Chdir(oldwd)
-
-				os.Chdir(tmpdir)
+				// No explicit image path specified. Will use temp dir as working directory,
+				// so we pull into a clean location.
+				tt.workDir = tmpdir
+				imageName := getImageNameFromURI(tt.srcURI)
+				tt.expectedImage = filepath.Join(tmpdir, imageName)
 
 				// if there's a pullDir, that's where we expect to find the image
 				if tt.pullDir != "" {
-					os.Chdir(tt.pullDir)
+					tt.expectedImage = filepath.Join(tt.pullDir, imageName)
 				}
 
-				tt.expectedImage = getImageNameFromURI(tt.srcURI)
 			}
 
 			// In order to actually test force, there must already be a file present in
@@ -467,7 +466,7 @@ func orasPushNoCheck(path, ref, layerMediaType string) error {
 }
 
 func (c ctx) testPullDisableCacheCmd(t *testing.T) {
-	cacheDir, err := ioutil.TempDir("", "e2e-imgcache-")
+	cacheDir, err := os.MkdirTemp("", "e2e-imgcache-")
 	if err != nil {
 		t.Fatalf("failed to create temporary directory: %s", err)
 	}
@@ -478,7 +477,7 @@ func (c ctx) testPullDisableCacheCmd(t *testing.T) {
 		}
 	}()
 
-	c.env.ImgCacheDir = cacheDir
+	c.env.UnprivCacheDir = cacheDir
 
 	disableCacheTests := []struct {
 		name      string
@@ -490,11 +489,6 @@ func (c ctx) testPullDisableCacheCmd(t *testing.T) {
 			imagePath: filepath.Join(c.env.TestDir, "library.sif"),
 			imageSrc:  "library://alpine:latest",
 		},
-		{
-			name:      "docker",
-			imagePath: filepath.Join(c.env.TestDir, "docker.sif"),
-			imageSrc:  "docker://alpine:latest",
-		},
 		{
 			name:      "oras",
 			imagePath: filepath.Join(c.env.TestDir, "oras.sif"),
@@ -529,7 +523,7 @@ func (c ctx) testPullDisableCacheCmd(t *testing.T) {
 }
 
 // testPullUmask will run some pull tests with different umasks, and
-// ensure the output file hase the correct permissions.
+// ensure the output file has the correct permissions.
 func (c ctx) testPullUmask(t *testing.T) {
 	umask22Image := "0022-umask-pull"
 	umask77Image := "0077-umask-pull"
@@ -640,23 +634,24 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		env: env,
 	}
 
-	// FIX: should run in parallel but the use of Chdir conflicts
-	// with other tests and can lead to test failures
-	return testhelper.Tests{
-		"ordered": testhelper.NoParallel(func(t *testing.T) {
-			// Run the tests the do not require setup.
-			t.Run("pullUmaskCheck", c.testPullUmask)
+	np := testhelper.NoParallel
 
+	return testhelper.Tests{
+		// Run pull tests sequentially among themselves, as they perform a lot
+		// of un-cached pulls which could otherwise lead to hitting rate limits.
+		"ordered": func(t *testing.T) {
 			// Setup a test registry to pull from (for oras).
 			c.setup(t)
-
 			t.Run("pull", c.testPullCmd)
 			t.Run("pullDisableCache", c.testPullDisableCacheCmd)
 			t.Run("concurrencyConfig", c.testConcurrencyConfig)
 			t.Run("concurrentPulls", c.testConcurrentPulls)
-
-			// Regressions
-			t.Run("issue5808", c.issue5808)
-		}),
+		},
+		// Manipulates umask for the process, so must be run alone to avoid
+		// causing permission issues for other tests.
+		"pullUmaskCheck": np(c.testPullUmask),
+		// Regressions
+		// Manipulates remotes, so must run alone
+		"issue5808": np(c.issue5808),
 	}
 }
